'''
Created on Nov 27, 2017

@author: Charlie McLouth
'''
from flask_restful import Resource, abort
from flask import request
from p4rest.flask.util import require_p4session
from p4rest.command import Command
from p4rest.exceptions import P4RestCommandError
import logging

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

class P4Commands(Resource):
    '''documentation'''

    def parseArguments(self, p4api):
        '''
        documentation
        
        If the method is GET or DELETE then we will ignore any post data
        If the method is PUT or POST then we will only process special query 
        string arguments. limit, offset, or attributes of the p4api. 
            for example: ?client=someclient
        '''
        logger.debug("parseArguments() enter")
        logger.debug(request.args)
        logger.debug(request.get_json())
        command_args = []
        command_kwargs = {}
        for k, v in request.args.lists():
            # convert lists of one member to non-list
            if isinstance(v, list) and len(v) == 1:
                v = v[0]
            # special arguments for the p4api
            if k in ["limit", "offset"] or hasattr(p4api, k):
                command_kwargs[k] = v
            # only accept if GET or DELETE
            if request.method in ["GET", "DELETE"]:
                # positional arguments
                if k == "arg":
                    if isinstance(v, list):
                        command_args = command_args + v
                    else:
                        command_args.append(v)
                # long form arguments
                elif k.startswith("--"):
                    command_args.append(k)
                    if v is not None and len(v) > 0:
                        if isinstance(v, list):
                            command_args = command_args + v
                        else:
                            command_args.append(v)
        # only process post data if PUT or POST
        if request.method in ["PUT", "POST"]:
            if request.is_json:
                data = request.get_json()
                if isinstance(data, (list, int, str)):
                    command_args = command_args + data
                elif isinstance(data, dict):
                    k = "arg"
                    if k in data:
                        command_args = command_args + data.pop(k)
                    k = "limit"
                    if k in data:
                        command_kwargs[k] = data.pop(k)
                    k = "offset"
                    if k in data:
                        command_kwargs[k] = data.pop(k)
                    command_args.append(data)
        logger.debug(command_args)
        logger.debug(command_kwargs)
        logger.debug("parseArguments() exit")
        return command_args, command_kwargs

    def validateCommand(self, session, commandwrapper, *args, **kwargs):
        '''documentation'''
        logger.debug("validateCommand() enter")
        try:
            commandtype = commandwrapper.validateCommand(None, *args, **kwargs)
        except P4RestCommandError as e:
            abort(e.httpstatuscode)
        if commandtype > Command.CT_LAST \
                or commandtype < Command.CT_FIRST:
            abort(404)
        # post command should only be save_ or run_
        elif request.method == "POST" \
                and commandtype not in [Command.CT_SAVE, 
                                        Command.CT_RUN]:
            abort(404)
        # get should not be delete_, save_, or update_
        elif request.method == "GET" \
                and commandtype in [Command.CT_DELETE, 
                                    Command.CT_SAVE, 
                                    Command.CT_UPDATE]:
            abort(404)
        # delete should only be delete_
        elif request.method == "DELETE"\
                and commandtype != Command.CT_DELETE:
            abort(404)
        # put should only be update_
        elif request.method == "PUT" \
                and commandtype != Command.CT_UPDATE:
            abort(404)
        logger.debug("validateCommand() exit")
        return commandtype

    def processRequest(self, command, session):
        logger.debug("processRequest() enter")
        commandwrapper = Command(session.getP4API(), command)
        args, kwargs = self.parseArguments(commandwrapper.p4api)
        commandtype = self.validateCommand(session, commandwrapper, *args, **kwargs)
        try:
            result = commandwrapper.executeCommandtype(commandtype, *args, **kwargs)
        except P4RestCommandError as e:
            abort(e.httpstatuscode)
        logger.debug("processRequest() exit")
        return result

    @require_p4session
    def post(self, command=None, **kwargs):
        '''documentation'''
        return self.processRequest(command, kwargs.get("current_session", None))

    @require_p4session
    def get(self, command=None, **kwargs):
        '''documentation'''
        return self.processRequest(command, kwargs.get("current_session", None))

    @require_p4session
    def put(self, command=None, **kwargs):
        '''documentation'''
        return self.processRequest(command, kwargs.get("current_session", None))

    @require_p4session
    def delete(self, command=None, **kwargs):
        '''documentation'''
        return self.processRequest(command, kwargs.get("current_session", None))

