'''
Created on Nov 21, 2017

@author: Charlie McLouth
'''
from flask import request, g
from flask_restful import Resource, abort
from p4rest.flask.util import require_p4session, require_adminsession, \
    require_auth
from p4rest.session import P4RestSession, SESSION_HEADER_NAME
from flask.globals import current_app
import os.path
from datetime import datetime
import logging

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)


class P4Sessions(Resource):

    @require_auth
    def post(self, P4PORT=None, P4USER=None, P4PASSWORD=None, **kwargs):
        sessionData = {"P4USER": P4USER, 
                       "P4PORT": P4PORT,
                       "P4CLIENT": current_app.config["P4CLIENT"],
                       "P4TICKETS": current_app.config["P4TICKETS"],
                       "P4TRUST": current_app.config["P4TRUST"],
                       "MAXRESULTS": current_app.config.get("MAXRESULTS", 0),
                       "SESSIONTYPE": "P4RestSessionP4"}
        publickey = P4RestSession.hashkey(datetime.utcnow().timestamp(),
                                          P4PORT, P4USER, P4PASSWORD)
        clientip = request.headers.get('X-Forwarded-For', 
                                          request.remote_addr)
        if clientip is not None:
            clientip = clientip.split(",")[0].strip()
        sessionData["SESSIONID"] = publickey
        privatekey = P4RestSession.hashPrivateKey(clientip, publickey)
        logger.debug("clientip:{};publickey:{};privatekey:{};".format(clientip, publickey, privatekey))
        sessionData["sessionfile"] = os.path.join(current_app.config["WORKDIR"],
                                                  privatekey,
                                                  current_app.config["P4CONFIG"])
        try:
            g.current_session = P4RestSession.createFromDict(sessionData)
            g.current_session.run_login(P4PASSWORD)
        except:
            # suppress
            message = "suppressing exception"
            logger.debug(message, exc_info=True)
            abort(401)
        result = {SESSION_HEADER_NAME: publickey}
        for key,value in g.current_session.items():
            if key in ["P4TICKETS", "P4TRUST"]:
                continue
            test = g.current_session.transformTypeToStr(key,value)
            if test is not None:
                result[key] = value
        for key,value in g.current_session.getP4API().run_info()[0].items():
            if key in ["clientCwd", "clientHost", "peerAddress", "clientAddress"]:
                continue
            result[key] = value
        return result

    @require_p4session
    def delete(self, sessionid=None, **kwargs):
        if sessionid is None or len(sessionid) < 1:
            abort(404)
        session = kwargs.get("current_session", None)
        if sessionid != session.get("SESSIONID", ""):
            abort(404)
        # confirm provided session is the same
        session.expire()
        return None, 204

    @require_p4session
    def get(self, sessionid=None, **kwargs):
        if sessionid is None or len(sessionid) < 1:
            abort(404)
        session = kwargs.get("current_session", None)
        if sessionid != session.get("SESSIONID", ""):
            abort(404)
        result = {SESSION_HEADER_NAME: sessionid}
        for key,value in g.current_session.items():
            if key in ["P4TICKETS", "P4TRUST"]:
                continue
            test = g.current_session.transformTypeToStr(key,value)
            if test is not None:
                result[key] = value
        for key,value in g.current_session.getP4API().run_info()[0].items():
            if key in ["clientCwd", "clientHost", "peerAddress", "clientAddress"]:
                continue
            result[key] = value
        return result

class AdminSessions(Resource):

    def post(self, *args, **kwargs):
        return "success"

    @require_adminsession
    def delete(self, sessionid=None, **kwargs):
        return "success"

    @require_adminsession
    def get(self, sessionid=None, **kwargs):
        return "success"
